Stan Math Library: stan/math/prim/scal/prob/double_exponential_log.hpp Source File

| Stan Math Library 2.10.0 reverse mode automatic differentiation

- Main Page
- Modules
- Namespaces
- Classes
- Files

- File List
- File Members

double_exponential_log.hpp
Go to the documentation of this file.
1 #ifndef STAN_MATH_PRIM_SCAL_PROB_DOUBLE_EXPONENTIAL_LOG_HPP
2 #define STAN_MATH_PRIM_SCAL_PROB_DOUBLE_EXPONENTIAL_LOG_HPP
3
4 #include <stan/math/prim/scal/meta/is_constant_struct.hpp>
5 #include <stan/math/prim/scal/meta/partials_return_type.hpp>
6 #include <stan/math/prim/scal/meta/OperandsAndPartials.hpp>
7 #include <stan/math/prim/scal/err/check_consistent_sizes.hpp>
8 #include <stan/math/prim/scal/err/check_finite.hpp>
9 #include <stan/math/prim/scal/err/check_not_nan.hpp>
10 #include <stan/math/prim/scal/err/check_positive_finite.hpp>
11 #include <stan/math/prim/scal/fun/value_of.hpp>
12 #include <stan/math/prim/scal/fun/log1m.hpp>
13 #include <stan/math/prim/scal/meta/contains_nonconstant_struct.hpp>
14 #include <stan/math/prim/scal/meta/VectorBuilder.hpp>
15 #include <stan/math/prim/scal/fun/constants.hpp>
16 #include <stan/math/prim/scal/meta/include_summand.hpp>
17 #include <stan/math/prim/scal/fun/sign.hpp>
18 #include <boost/random/uniform_01.hpp>
19 #include <boost/random/variate_generator.hpp>
20 #include <cmath>
21
22 namespace stan {
23
24 namespace math {
25
26 // DoubleExponential(y|mu, sigma) [sigma > 0]
27 // FIXME: add documentation
28 template <bool propto,
29 typename T_y, typename T_loc, typename T_scale>
30 typename return_type<T_y, T_loc, T_scale>::type
31 double_exponential_log(const T_y& y,
32 const T_loc& mu, const T_scale& sigma) {
33 static const char* function("stan::math::double_exponential_log");
34 typedef typename stan::partials_return_type<T_y, T_loc, T_scale>::type
35 T_partials_return;
36
37 using stan::is_constant_struct;
38 using stan::math::check_finite;
39 using stan::math::check_positive_finite;
40 using stan::math::check_consistent_sizes;
41 using stan::math::value_of;
42 using stan::math::include_summand;
43 using std::log;
44 using std::fabs;
45 using stan::math::sign;
46 using std::log;
47
48 // check if any vectors are zero length
49 if (!(stan::length(y)
50 && stan::length(mu)
51 && stan::length(sigma)))
52 return 0.0;
53
54 // set up return value accumulator
55 T_partials_return logp(0.0);
56 check_finite(function, "Random variable", y);
57 check_finite(function, "Location parameter", mu);
58 check_positive_finite(function, "Scale parameter", sigma);
59 check_consistent_sizes(function,
60 "Random variable", y,
61 "Location parameter", mu,
62 "Shape parameter", sigma);
63
64 // check if no variables are involved and prop-to
65 if (!include_summand<propto, T_y, T_loc, T_scale>::value)
66 return 0.0;
67
68 // set up template expressions wrapping scalars into vector views
69 VectorView<const T_y> y_vec(y);
70 VectorView<const T_loc> mu_vec(mu);
71 VectorView<const T_scale> sigma_vec(sigma);
72 size_t N = max_size(y, mu, sigma);
73 OperandsAndPartials<T_y, T_loc, T_scale>
74 operands_and_partials(y, mu, sigma);
75
76 VectorBuilder<include_summand<propto, T_y, T_loc, T_scale>::value,
77 T_partials_return, T_scale> inv_sigma(length(sigma));
78 VectorBuilder<!is_constant_struct<T_scale>::value,
79 T_partials_return, T_scale>
80 inv_sigma_squared(length(sigma));
81 VectorBuilder<include_summand<propto, T_scale>::value,
82 T_partials_return, T_scale> log_sigma(length(sigma));
83 for (size_t i = 0; i < length(sigma); i++) {
84 const T_partials_return sigma_dbl = value_of(sigma_vec[i]);
85 if (include_summand<propto, T_y, T_loc, T_scale>::value)
86 inv_sigma[i] = 1.0 / sigma_dbl;
87 if (include_summand<propto, T_scale>::value)
88 log_sigma[i] = log(value_of(sigma_vec[i]));
89 if (!is_constant_struct<T_scale>::value)
90 inv_sigma_squared[i] = inv_sigma[i] * inv_sigma[i];
91 }
92
93
94 for (size_t n = 0; n < N; n++) {
95 const T_partials_return y_dbl = value_of(y_vec[n]);
96 const T_partials_return mu_dbl = value_of(mu_vec[n]);
97
98 // reusable subexpressions values
99 const T_partials_return y_m_mu = y_dbl - mu_dbl;
100 const T_partials_return fabs_y_m_mu = fabs(y_m_mu);
101
102 // log probability
103 if (include_summand<propto>::value)
104 logp += NEG_LOG_TWO;
105 if (include_summand<propto, T_scale>::value)
106 logp -= log_sigma[n];
107 if (include_summand<propto, T_y, T_loc, T_scale>::value)
108 logp -= fabs_y_m_mu * inv_sigma[n];
109
110 // gradients
111 T_partials_return sign_y_m_mu_times_inv_sigma(0);
112 if (contains_nonconstant_struct<T_y, T_loc>::value)
113 sign_y_m_mu_times_inv_sigma = sign(y_m_mu) * inv_sigma[n];
114 if (!is_constant_struct<T_y>::value) {
115 operands_and_partials.d_x1[n] -= sign_y_m_mu_times_inv_sigma;
116 }
117 if (!is_constant_struct<T_loc>::value) {
118 operands_and_partials.d_x2[n] += sign_y_m_mu_times_inv_sigma;
119 }
120 if (!is_constant_struct<T_scale>::value)
121 operands_and_partials.d_x3[n] += -inv_sigma[n] + fabs_y_m_mu
122 * inv_sigma_squared[n];
123 }
124 return operands_and_partials.value(logp);
125 }
126
127
128 template <typename T_y, typename T_loc, typename T_scale>
129 typename return_type<T_y, T_loc, T_scale>::type
130 double_exponential_log(const T_y& y, const T_loc& mu,
131 const T_scale& sigma) {
132 return double_exponential_log<false>(y, mu, sigma);
133 }
134 }
135 }
136 #endif
check_not_nan.hpp
stan::math::OperandsAndPartials::d_x2
VectorView< T_return_type, false, true > d_x2
Definition: OperandsAndPartials.hpp:40
stan::math::fabs
fvar< T > fabs(const fvar< T > &x)
Definition: fabs.hpp:14
check_positive_finite.hpp
stan
Definition: log_sum_exp.hpp:8
stan::math::value_of
T value_of(const fvar< T > &v)
Return the value of the specified variable.
Definition: value_of.hpp:16
stan::math::sign
int sign(const T &z)
Definition: sign.hpp:9
stan::math::log
fvar< T > log(const fvar< T > &x)
Definition: log.hpp:15
stan::math::OperandsAndPartials::value
T_return_type value(double value)
Returns a T_return_type with the value specified with the partial derivatves.
Definition: OperandsAndPartials.hpp:68
contains_nonconstant_struct.hpp
stan::length
size_t length(const std::vector< T > &x)
Definition: length.hpp:10
is_constant_struct.hpp
stan::math::include_summand
Template metaprogram to calculate whether a summand needs to be included in a proportional (log) prob...
Definition: include_summand.hpp:36
stan::return_type::type
boost::math::tools::promote_args< typename scalar_type< T1 >::type, typename scalar_type< T2 >::type, typename scalar_type< T3 >::type, typename scalar_type< T4 >::type, typename scalar_type< T5 >::type, typename scalar_type< T6 >::type >::type type
Definition: return_type.hpp:27
include_summand.hpp
stan::is_constant_struct
Metaprogram to determine if a type has a base scalar type that can be assigned to type double...
Definition: is_constant_struct.hpp:13
stan::math::OperandsAndPartials
This class builds partial derivatives with respect to a set of operands.
Definition: OperandsAndPartials.hpp:38
stan::math::OperandsAndPartials::d_x3
VectorView< T_return_type, false, true > d_x3
Definition: OperandsAndPartials.hpp:41
stan::math::double_exponential_log
return_type< T_y, T_loc, T_scale >::type double_exponential_log(const T_y &y, const T_loc &mu, const T_scale &sigma)
Definition: double_exponential_log.hpp:31
stan::max_size
size_t max_size(const T1 &x1, const T2 &x2)
Definition: max_size.hpp:9
VectorBuilder.hpp
log1m.hpp
partials_return_type.hpp
OperandsAndPartials.hpp
stan::contains_nonconstant_struct
Definition: contains_nonconstant_struct.hpp:14
stan::math::check_finite
bool check_finite(const char *function, const char *name, const T_y &y)
Return true if y is finite.
Definition: check_finite.hpp:62
stan::math::NEG_LOG_TWO
const double NEG_LOG_TWO
Definition: constants.hpp:181
stan::VectorBuilder
VectorBuilder allocates type T1 values to be used as intermediate values.
Definition: VectorBuilder.hpp:28
constants.hpp
sign.hpp
stan::math::check_consistent_sizes
bool check_consistent_sizes(const char *function, const char *name1, const T1 &x1, const char *name2, const T2 &x2)
Return true if the dimension of x1 is consistent with x2.
Definition: check_consistent_sizes.hpp:31
check_finite.hpp
value_of.hpp
stan::VectorView
VectorView is a template expression that is constructed with a container or scalar, which it then allows to be used as an array using operator[].
Definition: VectorView.hpp:48
stan::partials_return_type::type
boost::math::tools::promote_args< typename partials_type< typename scalar_type< T1 >::type >::type, typename partials_type< typename scalar_type< T2 >::type >::type, typename partials_type< typename scalar_type< T3 >::type >::type, typename partials_type< typename scalar_type< T4 >::type >::type, typename partials_type< typename scalar_type< T5 >::type >::type, typename partials_type< typename scalar_type< T6 >::type >::type >::type type
Definition: partials_return_type.hpp:26
check_consistent_sizes.hpp
stan::math::check_positive_finite
bool check_positive_finite(const char *function, const char *name, const T_y &y)
Return true if y is positive and finite.
Definition: check_positive_finite.hpp:28
stan::math::OperandsAndPartials::d_x1
VectorView< T_return_type, false, true > d_x1
Definition: OperandsAndPartials.hpp:39

[ Stan Home Page ] © 2011–2016, Stan Development Team.